#ifndef Fpu_End
goto Fpu_End

'==============================================================================
' Copyright  Micromega Corporation 2004-2009. All rights reserved
'
' @file		fpuV3-i2c.bas
' @target	BasicATOM
'
' This file provides support routines for using the uM-FPU V3 floating point
' coprocessor with the BasicATOM microcontroller. It supports the I2C
' interface. (fpuV3-spi.bas is also available for supported the SPI interface.)
' This file is added to the main program using an include statement. The user
' must set the definitions for the interface pins appropriately (see below).
'
' @author Cam Thompson, Micromega Corporation
' @version
'	December 23, 2009
'	- updated for BMIDE 1.0.0.15, I2C syntax change
'	November 18, 2009
'	- ported to BasicATOM
'==============================================================================

' Pin assignments for the SPI interface must be added the main program.
' An example of the pin assignments is shown below:
'-------------------- uM-FPU pin definitions ---------------------------------
Fpu_SCL		con		P0				' I2C SCL (uM-FPU SOUT/SCL)
Fpu_SDA		con		P1				' I2C SDA (uM-FPU SIN/SDA)
Fpu_ID		con     $C8				' uM-FPU I2C device address

SBUF_SIZE	con		20				' size of string buffer

'-------------------- uM-FPU constants ---------------------------------------

READ_DELAY		con		15			' read setup delay

;-------------------- uM-FPU V3.1 opcode definitions -------------------------

SELECTA			con		$01		' Select register A
SELECTX			con		$02		' Select register X
CLR				con		$03		' reg[nn] = 0
CLRA			con		$04		' reg[A] = 0
CLRX			con		$05		' reg[X] = 0, X = X + 1
CLR0			con		$06		' reg[0] = 0
COPY			con		$07		' reg[nn] = reg[mm]
COPYA			con		$08		' reg[nn] = reg[A]
COPYX			con		$09		' reg[nn] = reg[X], X = X + 1
LOAD			con		$0A		' reg[0] = reg[nn]
LOADA			con		$0B		' reg[0] = reg[A]
LOADX			con		$0C		' reg[0] = reg[X], X = X + 1
ALOADX			con		$0D		' reg[A] = reg[X], X = X + 1
XSAVE			con		$0E		' reg[X] = reg[nn], X = X + 1
XSAVEA			con		$0F		' reg[X] = reg[A], X = X + 1
COPY0			con		$10		' reg[nn] = reg[0]
COPYI			con		$11		' reg[nn] = long(unsigned bb)
F_SWAP			con		$12		' Swap reg[nn] and reg[mm]
SWAPA			con		$13		' Swap reg[A] and reg[nn]
LEFT			con		$14		' Left parenthesis
RIGHT			con		$15		' Right parenthesis
FWRITE			con		$16		' Write 32-bit float to reg[nn]
FWRITEA			con		$17		' Write 32-bit float to reg[A]
FWRITEX			con		$18		' Write 32-bit float to reg[X], X = X + 1
FWRITE0			con		$19		' Write 32-bit float to reg[0]
FREAD			con		$1A		' Read 32-bit float from reg[nn]
FREADA			con		$1B		' Read 32-bit float from reg[A]
FREADX			con		$1C		' Read 32-bit float from reg[X], X = X + 1
FREAD0			con		$1D		' Read 32-bit float from reg[0]
ATOF			con		$1E		' Convert ASCII to float, store in reg[0]
FTOA			con		$1F		' Convert float to ASCII
FSET			con		$20		' reg[A] = reg[nn]
FADD			con		$21		' reg[A] = reg[A] + reg[nn]
FSUB			con		$22		' reg[A] = reg[A] - reg[nn]
FSUBR			con		$23		' reg[A] = reg[nn] - reg[A]
FMUL			con		$24		' reg[A] = reg[A] * reg[nn]
FDIV			con		$25		' reg[A] = reg[A] / reg[nn]
FDIVR			con		$26		' reg[A] = reg[nn] / reg[A]
FPOW			con		$27		' reg[A] = reg[A] ** reg[nn]
FCMP			con		$28		' Float compare reg[A] - reg[nn]
FSET0			con		$29		' reg[A] = reg[0]
FADD0			con		$2A		' reg[A] = reg[A] + reg[0]
FSUB0			con		$2B		' reg[A] = reg[A] - reg[0]
FSUBR0			con		$2C		' reg[A] = reg[0] - reg[A]
FMUL0			con		$2D		' reg[A] = reg[A] * reg[0]
FDIV0			con		$2E		' reg[A] = reg[A] / reg[0]
FDIVR0			con		$2F		' reg[A] = reg[0] / reg[A]
FPOW0			con		$30		' reg[A] = reg[A] ** reg[0]
FCMP0			con		$31		' Float compare reg[A] - reg[0]
FSETI			con		$32		' reg[A] = float(bb)
FADDI			con		$33		' reg[A] = reg[A] + float(bb)
FSUBI			con		$34		' reg[A] = reg[A] - float(bb)
FSUBRI			con		$35		' reg[A] = float(bb) - reg[A]
FMULI			con		$36		' reg[A] = reg[A] * float(bb)
FDIVI			con		$37		' reg[A] = reg[A] / float(bb)
FDIVRI			con		$38		' reg[A] = float(bb) / reg[A]
FPOWI			con		$39		' reg[A] = reg[A] ** bb
FCMPI			con		$3A		' Float compare reg[A] - float(bb)
FSTATUS			con		$3B		' Float status of reg[nn]
FSTATUSA		con		$3C		' Float status of reg[A]
FCMP2			con		$3D		' Float compare reg[nn] - reg[mm]
FNEG			con		$3E		' reg[A] = -reg[A]
FABS			con		$3F		' reg[A] = | reg[A] |
FINV			con		$40		' reg[A] = 1 / reg[A]
SQRT			con		$41		' reg[A] = sqrt(reg[A])
ROOT			con		$42		' reg[A] = root(reg[A], reg[nn])
LOG				con		$43		' reg[A] = log(reg[A])
LOG10			con		$44		' reg[A] = log10(reg[A])
EXP				con		$45		' reg[A] = exp(reg[A])
EXP10			con		$46		' reg[A] = exp10(reg[A])
F_SIN			con		$47		' reg[A] = sin(reg[A])
F_COS			con		$48		' reg[A] = cos(reg[A])
TAN				con		$49		' reg[A] = tan(reg[A])
ASIN			con		$4A		' reg[A] = asin(reg[A])
ACOS			con		$4B		' reg[A] = acos(reg[A])
ATAN			con		$4C		' reg[A] = atan(reg[A])
ATAN2			con		$4D		' reg[A] = atan2(reg[A], reg[nn])
DEGREES			con		$4E		' reg[A] = degrees(reg[A])
RADIANS			con		$4F		' reg[A] = radians(reg[A])
FMOD			con		$50		' reg[A] = reg[A] MOD reg[nn]
FLOOR			con		$51		' reg[A] = floor(reg[A])
CEIL			con		$52		' reg[A] = ceil(reg[A])
ROUND			con		$53		' reg[A] = round(reg[A])
FMIN			con		$54		' reg[A] = min(reg[A], reg[nn])
FMAX			con		$55		' reg[A] = max(reg[A], reg[nn])
FCNV			con		$56		' reg[A] = conversion(nn, reg[A])
FMAC			con		$57		' reg[A] = reg[A] + (reg[nn] * reg[mm])
FMSC			con		$58		' reg[A] = reg[A] - (reg[nn] * reg[mm])
LOADBYTE		con		$59		' reg[0] = float(signed bb)
LOADUBYTE		con		$5A		' reg[0] = float(unsigned byte)
LOADWORD		con		$5B		' reg[0] = float(signed word)
LOADUWORD		con		$5C		' reg[0] = float(unsigned word)
LOADE			con		$5D		' reg[0] = 2.7182818
LOADPI			con		$5E		' reg[0] = 3.1415927
LOADCON			con		$5F		' reg[0] = float constant(nn)
F_FLOAT			con		$60		' reg[A] = float(reg[A])
FIX				con		$61		' reg[A] = fix(reg[A])
FIXR			con		$62		' reg[A] = fix(round(reg[A]))
FRAC			con		$63		' reg[A] = fraction(reg[A])
FSPLIT			con		$64		' reg[A] = int(reg[A]), reg[0] = frac(reg[A])
SELECTMA		con		$65		' Select matrix A
SELECTMB		con		$66		' Select matrix B
SELECTMC		con		$67		' Select matrix C
LOADMA			con		$68		' reg[0] = matrix A[bb,bb]
LOADMB			con		$69		' reg[0] = matrix B[bb, bb]
LOADMC			con		$6A		' reg[0] = matrix C[bb, bb]
SAVEMA			con		$6B		' matrix A[bb,bb] = reg[A]
SAVEMB			con		$6C		' matrix B[bb,bb] = reg[A]
SAVEMC			con		$6D		' matrix C[bb,bb] = reg[A]
MOP				con		$6E		' Matrix operation
FFT				con		$6F		' FFT operation
WRBLK			con		$70		' write register block
RDBLK			con		$71		' read register block
LOADIND			con		$7A		' reg[0] = reg[reg[nn]]
SAVEIND			con		$7B		' reg[reg[nn]] = reg[A]
INDA			con		$7C		' Select A using reg[nn]
INDX			con		$7D		' Select X using reg[nn]
FCALL			con		$7E		' Call function in Flash memory
EECALL			con		$7F		' Call function in EEPROM memory
RET				con		$80		' Return from function
BRA				con		$81		' Unconditional branch
BRACC			con		$82		' Conditional branch
JMP				con		$83		' Unconditional jump
JMPCC			con		$84		' Conditional jump
TABLE			con		$85		' Table lookup
FTABLE			con		$86		' Floating point reverse table lookup
LTABLE			con		$87		' Long integer reverse table lookup
POLY			con		$88		' reg[A] = nth order polynomial
F_GOTO			con		$89		' Computed goto
RETCC			con		$8A		' Conditional return from function
LWRITE			con		$90		' Write 32-bit long integer to reg[nn]
LWRITEA			con		$91		' Write 32-bit long integer to reg[A]
LWRITEX			con		$92		' Write 32-bit long integer to reg[X], X = X + 1
LWRITE0			con		$93		' Write 32-bit long integer to reg[0]
LREAD			con		$94		' Read 32-bit long integer from reg[nn]
LREADA			con		$95		' Read 32-bit long integer from reg[A]
LREADX			con		$96		' Read 32-bit long integer from reg[X], X = X + 1
LREAD0			con		$97		' Read 32-bit long integer from reg[0]
LREADBYTE		con		$98		' Read lower 8 bits of reg[A]
LREADWORD		con		$99		' Read lower 16 bits reg[A]
ATOL			con		$9A		' Convert ASCII to long integer
LTOA			con		$9B		' Convert long integer to ASCII
LSET			con		$9C		' reg[A] = reg[nn]
LADD			con		$9D		' reg[A] = reg[A] + reg[nn]
LSUB			con		$9E		' reg[A] = reg[A] - reg[nn]
LMUL			con		$9F		' reg[A] = reg[A] * reg[nn]
LDIV			con		$A0		' reg[A] = reg[A] / reg[nn]
LCMP			con		$A1		' Signed long compare reg[A] - reg[nn]
LUDIV			con		$A2		' reg[A] = reg[A] / reg[nn]
LUCMP			con		$A3		' Unsigned long compare of reg[A] - reg[nn]
LTST			con		$A4		' Long integer status of reg[A] AND reg[nn] 
LSET0			con		$A5		' reg[A] = reg[0]
LADD0			con		$A6		' reg[A] = reg[A] + reg[0]
LSUB0			con		$A7		' reg[A] = reg[A] - reg[0]
LMUL0			con		$A8		' reg[A] = reg[A] * reg[0]
LDIV0			con		$A9		' reg[A] = reg[A] / reg[0]
LCMP0			con		$AA		' Signed long compare reg[A] - reg[0]
LUDIV0			con		$AB		' reg[A] = reg[A] / reg[0]
LUCMP0			con		$AC		' Unsigned long compare reg[A] - reg[0]
LTST0			con		$AD		' Long integer status of reg[A] AND reg[0] 
LSETI			con		$AE		' reg[A] = long(bb)
LADDI			con		$AF		' reg[A] = reg[A] + long(bb)
LSUBI			con		$B0		' reg[A] = reg[A] - long(bb)
LMULI			con		$B1		' reg[A] = reg[A] * long(bb)
LDIVI			con		$B2		' reg[A] = reg[A] / long(bb)
LCMPI			con		$B3		' Signed long compare reg[A] - long(bb)
LUDIVI			con		$B4		' reg[A] = reg[A] / unsigned long(bb)
LUCMPI			con		$B5		' Unsigned long compare reg[A] - ulong(bb)
LTSTI			con		$B6		' Long integer status of reg[A] AND ulong(bb)
LSTATUS			con		$B7		' Long integer status of reg[nn]
LSTATUSA		con		$B8		' Long integer status of reg[A]
LCMP2			con		$B9		' Signed long compare reg[nn] - reg[mm]
LUCMP2			con		$BA		' Unsigned long compare reg[nn] - reg[mm]
LNEG			con		$BB		' reg[A] = -reg[A]
LABS			con		$BC		' reg[A] = | reg[A] |
LINC			con		$BD		' reg[nn] = reg[nn] + 1
LDEC			con		$BE		' reg[nn] = reg[nn] - 1
LNOT			con		$BF		' reg[A] = NOT reg[A]
LAND			con		$C0		' reg[A] = reg[A] AND reg[nn]
LOR				con		$C1		' reg[A] = reg[A] OR reg[nn]
LXOR			con		$C2		' reg[A] = reg[A] XOR reg[nn]
LSHIFT			con		$C3		' reg[A] = reg[A] shift reg[nn]
LMIN			con		$C4		' reg[A] = min(reg[A], reg[nn])
LMAX			con		$C5		' reg[A] = max(reg[A], reg[nn])
LONGBYTE		con		$C6		' reg[0] = long(signed byte bb)
LONGUBYTE		con		$C7		' reg[0] = long(unsigned byte bb)
LONGWORD		con		$C8		' reg[0] = long(signed word wwww)
LONGUWORD		con		$C9		' reg[0] = long(unsigned word wwww)
SETSTATUS		con		$CD		' Set status byte
F_SEROUT		con		$CE		' Serial output
F_SERIN			con		$CF		' Serial Input
SETOUT			con		$D0		' Set OUT1 and OUT2 output pins
ADCMODE			con		$D1		' Set A/D trigger mode
ADCTRIG			con		$D2		' A/D manual trigger
ADCSCALE		con		$D3		' ADCscale[ch] = B
ADCLONG			con		$D4		' reg[0] = ADCvalue[ch]
ADCLOAD			con		$D5		' reg[0] = float(ADCvalue[ch]) * ADCscale[ch]
ADCWAIT			con		$D6		' wait for next A/D sample
TIMESET			con		$D7		' time = reg[0]
TIMELONG		con		$D8		' reg[0] = time (long)
TICKLONG		con		$D9		' reg[0] = ticks (long)
EESAVE			con		$DA		' EEPROM[nn] = reg[mm]
EESAVEA			con		$DB		' EEPROM[nn] = reg[A]
EELOAD			con		$DC		' reg[nn] = EEPROM[mm]
EELOADA			con		$DD		' reg[A] = EEPROM[nn]
EEWRITE			con		$DE		' Store bytes in EEPROM
EXTSET			con		$E0		' external input count = reg[0]
EXTLONG			con		$E1		' reg[0] = external input counter (long)
EXTWAIT			con		$E2		' wait for next external input
STRSET			con		$E3		' Copy string to string buffer
STRSEL			con		$E4		' Set selection point
STRINS			con		$E5		' Insert string at selection point
STRCMP			con		$E6		' Compare string with string buffer
STRFIND			con		$E7		' Find string and set selection point
STRFCHR			con		$E8		' Set field separators
STRFIELD		con		$E9		' Find field and set selection point
STRTOF			con		$EA		' Convert string selection to float
STRTOL			con		$EB		' Convert string selection to long
READSEL			con		$EC		' Read string selection
STRBYTE			con		$ED		' Insert 8-bit byte at selection point
STRINC			con		$EE		' increment selection point
STRDEC			con		$EF		' decrement selection point
F_SYNC			con		$F0		' Get synchronization byte
READSTATUS		con		$F1		' Read status byte
READSTR			con		$F2		' Read string from string buffer
VERSION			con		$F3		' Copy version string to string buffer
IEEEMODE		con		$F4		' Set IEEE mode (default)
PICMODE			con		$F5		' Set PIC mode
CHECKSUM		con		$F6		' Calculate checksum for uM-FPU code
BREAK			con		$F7		' Debug breakpoint
TRACEOFF		con		$F8		' Turn debug trace off
TRACEON			con		$F9		' Turn debug trace on
TRACESTR		con		$FA		' Send string to debug trace buffer
TRACEREG		con		$FB		' Send register value to trace buffer
READVAR			con		$FC		' Read internal variable, store in reg[0]

SYNC_CHAR		con		$5C		' sync character

;-------------------- uM-FPU V3.1 MOP definitions -----------------------------

SCALAR_SET		con		0		' MA[row,col] = reg[0], for all elements
SCALAR_ADD		con		1		' MA[row,col] = MA[row,col] + reg[0], for all elements
SCALAR_SUB		con		2		' MA[row,col] = reg[0] - MA[row,col], for all elements
SCALAR_SUBR		con		3		' MA[row,col] = MA[row,col] + reg[0], for all elements
SCALAR_MUL		con		4		' MA[row,col] = MA[row,col] * reg[0], for all elements
SCALAR_DIV		con		5		' MA[row,col] = MA[row,col] / reg[0], for all elements
SCALAR_DIVR		con		6		' MA[row,col] = reg[0] / MA[row,col], for all elements
SCALAR_POW		con		7		' MA[row,col] = MA[row,col] ** reg[0], for all elements
EWISE_SET		con		8		' MA[row,col] = MB[row,col], for all elements
EWISE_ADD		con		9		' MA[row,col] = MA[row,col] + MB[row,col], for all elements
EWISE_SUB		con		10		' MA[row,col] = MB[row,col] - MA[row,col], for all elements
EWISE_SUBR		con		11		' MA[row,col] = MA[row,col] + MB[row,col], for all elements
EWISE_MUL		con		12		' MA[row,col] = MA[row,col] * MB[row,col], for all elements
EWISE_DIV		con		13		' MA[row,col] = MA[row,col] / MB[row,col], for all elements
EWISE_DIVR		con		14		' MA[row,col] = MB[row,col] / MA[row,col], for all elements
EWISE_POW		con		15		' MA[row,col] = MA[row,col] ** MB[row,col], for all elements
MX_MULTIPLY		con		16		' MA = MB * MC matrix multiply
MX_IDENTITY		con		17		' MA = identity matrix
MX_DIAGONAL		con		18		' MA = diagonal matrix
MX_TRANSPOSE	con		19		' MA = transpose of MB
MX_COUNT		con		20		' reg[0] = number of elements in MA
MX_SUM			con		21		' reg[0] = sum of all elements in MA
MX_AVE			con		22		' reg[0] = average value of all elements in MA
MX_MIN			con		23		' reg[0] = minimum value of all elements in MA
MX_MAX			con		24		' reg[0] = maximum value of all elements in MA
MX_COPYAB		con		25		' copy MA to MC
MX_COPYAC		con		26		' copy MA to MC
MX_COPYBA		con		27		' copy MB to MA
MX_COPYBC		con		28		' copy MB to MC
MX_COPYCA		con		29		' copy MC to MA
MX_COPYCB		con		30		' copy MC to MB
MX_DETERM		con		31		' reg0 = determinant of MA (2x2 and 3x3)
MX_INVERSE		con		32		' MA = inverse of MB (2x2 and 3x3)
MX_ILOADRA		con		33		' indexed load registers to MA
MX_ILOADRB		con		34		' indexed load registers to MB
MX_ILOADRC		con		35		' indexed load registers to MC
MX_ILOADBA		con		36		' indexed load MB to MA
MX_ILOADCA		con		37		' indexed load MC to MA
MX_ISAVEAR		con		38		' indexed save MA to registers
MX_ISAVEAB		con		39		' indexed save MA to MB
MX_ISAVEAC		con		40		' indexed save MA to MC

;-------------------- uM-FPU V3.1 FFT definitions -----------------------------

FIRST_STAGE		con		0		' first stage of FFT
NEXT_STAGE		con		1		' next stage of FFT
NEXT_LEVEL		con		2		' next level of FFT
NEXT_BLOCK		con		3		' next block of FFT
BIT_REVERSE		con		4		' bit reverse sort (pre-processing)
PRE_ADJUST		con		8		' inverse FFT (pre-processing)
POST_ADJUST		con		16		' inverse FFT (post-processing)

'-------------------- uM-FPU variables ---------------------------------------

dataWord		var		word				' data word
dataHigh		var		dataWord.HIGHBYTE	' high byte of dataWord
dataLow			var		dataword.LOWBYTE	' low byte of dataLow
dataByte		var		dataLow				' (alternate name)
opcode			var		dataHigh			' opcode (same as dataHigh)
reg				var		dataHigh			' register (same as dataHigh)
format			var		dataLow				' format (same as dataLow)
fpu_status		var		dataLow				' status (same as dataLow)
sbuf			var		byte(SBUF_SIZE)		' string buffer

fpu_status_Zero	var		fpu_status.BIT0		' Zero status bit (0-not zero, 1-zero)
fpu_status_Sign	var		fpu_status.BIT1		' Sign status bit (0-positive, 1-negative)
fpu_status_NaN	var		fpu_status.BIT2		' Not a Number status bit (0-valid number, 1-NaN)
fpu_status_Inf	var		fpu_status.BIT3		' Infinity status bit (0-not infinite, 1-infinite)

;-------------------- Fpu_Reset ----------------------------------------------

Fpu_Reset:								' send reset command to uM-FPU V3
	i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [1, 0]
	pause 10
										' check for synchronization
	i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0, F_SYNC]
	goto Fpu_Status2

;-------------------- Fpu_Wait -----------------------------------------------

Fpu_Wait:
	i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0]
	i2cin Fpu_SDA, Fpu_SCL, Fpu_ID, [fpu_status]
	if fpu_status <> 0 then Fpu_Wait
	return

;-------------------- Fpu_ReadStatus -----------------------------------------

Fpu_ReadStatus:							' read the status
	gosub Fpu_Wait
	i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0, READSTATUS]

Fpu_Status2:  
	pauseus READ_DELAY
	i2cin Fpu_SDA, Fpu_SCL, Fpu_ID, [fpu_status]
	return

;-------------------- Fpu_ReadDelay ------------------------------------------

Fpu_ReadDelay:  						' pause for read setup delay                  
	pauseus READ_DELAY
	return

;-------------------- Print_Version ------------------------------------------

Print_Version:							' get the uM-FPU version string
	i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0, VERSION]
	goto Print_FpuString

;-------------------- Print_Float, Print_FloatFormat -------------------------

Print_Float:							' set format to zero (free format)
	format = 0							' (fall through to Print_FloatFormat)

Print_FloatFormat:
	opcode = FTOA						' convert floating point to formatted ASCII
	goto Print_Format

;-------------------- Print_Long, Print_LongFormat ---------------------------

Print_Long:								'set format to 0 (free format)
  format = 0							' (fall through to Print_LongFormat)

Print_LongFormat:						' convert long integer to formatted ASCII
  opcode = LTOA							' (fall through to Print_Format)

Print_Format:							' send conversion command
	i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0, opcode, format]
										' (fall through to Print_FpuString)

;-------------------- Fpu_FpuString ------------------------------------------

Print_FpuString:
	gosub Fpu_Wait						' wait until uM-FPU V3 is ready
	i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0, READSTR]

Print_FpuString2:
	pauseus READ_DELAY
	i2cin Fpu_SDA, Fpu_SCL, Fpu_ID, [str sbuf\SBUF_SIZE\0]
	serout S_OUT, I19200, [str sbuf\SBUF_SIZE\0]
	return
	
Fpu_End:
#endif