'=============================================================================
' Copyright  Micromega Corporation 2005-2009. All rights reserved.
'
' @file		graph-spi.bsp
' @target	BasicATOM
'	
' This program demonstrates the use of the uM-FPU V3 floating point coprocessor
' with the BasicATOM microcontroller using an I2C interface. It draws a simple
' graph of the sine and cosine functions using the serout command.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	December 23, 2009
'	- updated for BMIDE 1.0.0.15, I2C syntax change
'	October 9, 2009
'	- ported to BasicATOM
'==============================================================================

'-------------------- uM-FPU pin definitions ----------------------------------

Fpu_SCLK		con		P0				' SPI SCLK (uM-FPU SCLK)
Fpu_SOUT		con		P1				' SPI MISO (uM-FPU SOUT)
Fpu_SIN			con		P2				' SPI MOSI (uM-FPU SIN)
Fpu_Busy		con		IN1				' FPU Busy/Ready status

#include "C:\My BasicATOM\fpuv3-i2c.bas"

'-------------------- uM-FPU register definitions -----------------------------
Angle           con     1               ' angle
Point           con     2               ' point on graph

'-------------------- variables ----------------------------------------------

cnt             var     byte            ' counter
tmp             var     byte            ' temporary value
x               var     byte            ' x value for graph
ipoint          var     byte            ' point value for graph

'=============================================================================
'-------------------- initialization -----------------------------------------
'=============================================================================

Reset:
	serout S_OUT, I19200, [13, 13, "Graph-SPI", 13]

	gosub Fpu_Reset						' reset the FPU hardware
	if fpu_status <> SYNC_CHAR then
		serout S_OUT, I19200, ["uM-FPU not detected: ", hex fpu_status]
		end
	else
		gosub Print_Version				' display the uM-FPU version number
		serout S_OUT, I19200, [13]
	endif

'=============================================================================
'-------------------- main routine -------------------------------------------
'=============================================================================

Main:
	' graph sine from 0 TO 360 degrees (2*PI radians)
	'------------------------------------------------
	serout S_OUT, I19200, [13, "Graph of sine"]
	serout S_OUT, I19200, [13, "-1        0         1"]
	serout S_OUT, I19200, [13, "---------------------", 13]
	for cnt = 0 TO 20
    	' Angle = sin(pi * 2 * cnt / 20)
    	'-------------------------------
    	gosub Fpu_Wait
  		i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0, SELECTA, Angle, LOADPI, FSET0]
    	dataByte = cnt*2
  		i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0, FMULI, dataByte, FDIVI, 20, F_SIN]
		gosub Graph
	next

	' graph cosine from 0 TO 360 degrees (2*PI radians)
	'--------------------------------------------------
	serout S_OUT, I19200, [13, "Graph of cosine"]
	serout S_OUT, I19200, [13, "-1        0         1"]
	serout S_OUT, I19200, [13, "---------------------", 13]
	for cnt = 0 TO 20
    	' Angle = cos(pi * 2 * cnt / 20)
    	'-------------------------------
    	gosub Fpu_Wait
		i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0, SELECTA, Angle, LOADPI, FSET0]
    	dataByte = cnt*2
    	i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0, FMULI, dataByte, FDIVI, 20, F_COS]
		gosub Graph
	next

Done: 
	serout S_OUT, I19200, [13, "Done.", 13]
	end

'-------------------- Graph --------------------------------------------------

Graph:
	' Point = Fix(Round(Angle * 10))
	'-------------------------------
	gosub Fpu_Wait
    i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0, SELECTA, Point, FSET, Angle, FMULI, 10, FIXR]

	' read the lower 8 bits of lonf value
	'------------------------------------
	gosub Fpu_Wait
    i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0, LREADBYTE]
	gosub Fpu_ReadDelay
	i2cin Fpu_SDA, Fpu_SCL, Fpu_ID, [ipoint]

	' add offset to midpoint of graph
	'--------------------------------
	ipoint = ipoint + 10

  ' plot the point on graph
  '------------------------
 
	for x = 0 TO 20						' plot the point on graph
		if (x = 10) then
			tmp = "|"
		else
			tmp = " "
		endif
		if (x = ipoint) then
			tmp = "*"
		endif
		serout S_OUT, I19200, [tmp]
	next

	' display the original angle in 8.5 format
	'-----------------------------------------
	serout S_OUT, I19200, [" "]
    i2cout Fpu_SDA, Fpu_SCL, Fpu_ID, [0, SELECTA, Angle]
	format = 85     					' 8.5 format i.e. width=8, decimals=5)
	gosub Print_FloatFormat
	serout S_OUT, I19200, [13]
	return
