'==============================================================================
' Copyright  Micromega Corporation 2002-2009. All rights reserved
'
' @file   GPSdemo1-spi.bas
' @target BasicATOM
'
' This demo program reads latitude and longitude values from a GPS receiver
' and displays them in the terminal window using the serout command. The 4800
' baud serial output from the GPS receiver is connected to the SERIN pin of
' the uM-FPU V3.1 chip. The sentence count and sentence type is displayed
' for each record, and the longitude and latitude values are displayed for the
' GPRMC sentence. The sentence types will vary depending on the GPS receiver
' used. The following example shows the output from a Garmin receiver:
'
'   GPSdemo1
'   uM-FPU V3.1.0
'
'   1:GPGLL
'   2:GPBOD
'   3:GPVTG
'   4:PGRME
'   5:PGRMZ
'   6:PGRMM
'   7:HCHDG
'   8:GPRTE
'   9:GPRMC: Lat = 7622.4609, Long = 7622.4609
'   (etc.)
' ...
'
' See Application Note 36 - Reading GPS Data
'
' @author Cam Thompson, Micromega Corporation
' @version
'	December 17, 2009
'	- updated for BMIDE 1.0.0.15, SPI now defaults to 8-bit
'	October 9, 2009
'	- ported to BasicATOM
'==============================================================================

'-------------------- include uM-FPU V3 support code --------------------------

#include "C:\My BasicATOM\fpuv3-spi.bas"

'-------------------- uM-FPU register definitions -----------------------------
Latitude        con     8				' latitude
Longitude       con     9				' longitude

'-------------------- variables -----------------------------------------------
cnt             var     Word            ' counter

'-------------------- definitions ---------------------------------------------
GPS_OK          con     $80             ' GPS sentence OK status

'==============================================================================
'-------------------- initialization ------------------------------------------
'==============================================================================

Reset:		
	serout S_OUT, I19200, [13, 13, "GPSdemo1", 13]

	gosub Fpu_Reset						' reset the FPU hardware
	if fpu_status <> SYNC_CHAR then
		serout S_OUT, I19200, ["uM-FPU not detected: ", hex fpu_status]
		end
	else
		gosub Print_Version				' display the uM-FPU version number
		serout S_OUT, I19200, [13]
	endif

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:
	shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [F_SEROUT, 0, 5]	' set 4800 baud
	shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [F_SERIN, 4]			' enable NMEA

	for cnt = 1 TO 200
		serout S_OUT, I19200, [13, DEC cnt, ":"]
    	shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [F_SERIN, 6]
		gosub Fpu_ReadStatus

		if fpu_status = GPS_OK then
			gosub Print_FpuStringSel

			shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [STRCMP, "GPRMC", 0]
			gosub Fpu_ReadStatus

			if fpu_status_Zero then
				shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [SELECTA, Latitude, STRFIELD, 4, STRTOF, FSET0]
				shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [SELECTA, Longitude, STRFIELD, 6, STRTOF, FSET0]
				shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [STRSET, 0]
  				shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [SELECTA, Latitude]
				serout S_OUT, I19200, [13, "*** Lat = "]
				gosub Print_Float
				shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [SELECTA, Longitude]
				serout S_OUT, I19200, [ ", Long = "]
				gosub Print_Float
			endif

		else
			serout S_OUT, I19200, [13, "*** data error ***"]
		endif
	next

Done:
	serout S_OUT, I19200, [13, "Done.", 13]
	end

'-------------------- print string selection ----------------------------------

Print_FpuStringSel:
	gosub Fpu_Wait						' wait until uM-FPU is ready
	shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [READSEL]
	gosub Fpu_ReadDelay
	goto Print_FpuStr2
  
