'==============================================================================
' Copyright  Micromega Corporation 2004-2009. All rights reserved.
'
' @file		sample-spi.bsp
' @target	BasicATOM
'
' Demonstration routine for uM-FPU V3 using an SPI interface.
' The demo program takes an integer value representing the diameter
' of a circle in centimeters, converts the value to inches, then calculates
' the circumference and area in inches and square inches.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	December 17, 2009
'	- updated for BMIDE 1.0.0.15, SPI now defaults to 8-bit
'	October 9, 2009
'	- ported to BasicATOM
'==============================================================================

'-------------------- include uM-FPU V3 support code --------------------------

#include "C:\My BasicATOM\fpuv3-spi.bas"

'-------------------- uM-FPU register definitions -----------------------------

DiameterIn		con		1			' (FPU register) diameter in inches
Circumference	con		2			' (FPU register) circumference
Area			con		3			' (FPU register) 
Pi				con		4			' (FPU register) constant pi

'-------------------- variables -----------------------------------------------

diameterCm		var		byte		' diameter in centimeters

;=============================================================================
;-------------------- main routine -------------------------------------------
;=============================================================================


	serout S_OUT, I19200, [13, 13, "Conversion Example"]
	serout S_OUT, I19200, [13, "------------------", 13]

	;------------------------------------
	; Reset the uM-FPU and print version.
	;------------------------------------

	gosub Fpu_Reset						' reset the FPU hardware
	if fpu_status <> SYNC_CHAR then
		serout S_OUT, I19200, ["uM-FPU not detected: ", hex fpu_status]
		end
	else
		gosub Print_Version				' display the uM-FPU version number
		serout S_OUT, I19200, [13]
	endif

	;-----------------------------------------------------
	; Load constant for later use.
	;-----------------------------------------------------
	shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [SELECTA, Pi, LOADPI, FSET0]


	;----------------------------------------------------------------------------
	; Get diameter in centimeters.  The value would typically come from a sensor
	; reading, but in this example an assumed value of 25 is used.
	;----------------------------------------------------------------------------
	diameterCm = 25
	serout S_OUT, I19200, [13, 13, "Diameter (cm):       ", DEC diameterCm]
	
	;------------------------------
	; diameterIn = diameterCm / 2.54
	;------------------------------
	shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [SELECTA, DiameterIn, FSETI, diameterCm, FCNV, 5]
	serout S_OUT, I19200, [13, "Diameter (in.):      "]
	gosub Print_Float

	;------------------------------
	; circumference = diameter * pi
	;------------------------------
	shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [SELECTA, Circumference, FSET, DiameterIn, FMUL, Pi]
	serout S_OUT, I19200, [13, "Circumference (in.): "]
	gosub Print_Float

	;-----------------------------
	; area = (diameter / 2)^2 * pi
	;-----------------------------
	shiftout Fpu_SIN, Fpu_SCLK, FASTMSBPRE, [SELECTA, Area, FSET, DiameterIn, FDIVI, 2, FMUL, Area, FMUL, Pi]
	serout S_OUT, I19200, [13, "Area (sq.in.):       "]
	gosub Print_Float

	serout S_OUT, I19200, [13, "Done.", 13]
	end

